#![allow(dead_code)]

use sparse::Sparse;
use pool::Pool;
use trie::Trie;
use crate::util::{
    Chain,
    pack::pack_u32,
};

pub mod connection; use connection::Connection;
pub mod user; use user::User;
pub mod authentication; use authentication::Authentication;
pub mod session; use session::{Session, SessionToken};
pub mod context;

pub struct App {
    pub connections:Pool<Connection>,

    pub users:Pool<User>,
    pub user_next:u32,
    pub user_id:Sparse<usize>,
    pub user_handle:Trie<u32>,
    pub salts:Vec<[u8; 16]>,

    pub auths:Trie<Authentication>,
    pub sessions:Trie<Session>,

    pub session_time:Chain<SessionToken>,
}
impl App {
    pub fn new() -> Self
    {
        Self {
            connections:Pool::new(),

            users:Pool::new(),
            user_next:0,
            user_id:Sparse::new(),
            user_handle:Trie::new(),
            salts:Vec::new(),

            auths:Trie::new(),
            sessions:Trie::new(),

            session_time:Chain::new(),
        }
    }

    pub fn init(&mut self) -> Result<(),std::io::Error>
    { use std::{path::Path, fs};
        
        let path_data = Path::new("data");
        if !path_data.exists() {
            fs::create_dir(path_data)?;
            fs::create_dir(path_data.join("c"))?;
            fs::create_dir(path_data.join("s"))?;
            fs::create_dir(path_data.join("u"))?;

            fs::write(path_data.join("c/.i"), pack_u32(0)).ok();
            fs::write(path_data.join("s/.i"), pack_u32(0)).ok();
            fs::write(path_data.join("u/.i"), pack_u32(0)).ok();
        }
        
        Ok(())
    }
}
